#ifdef SCCS
static char sccsid[]="@(#)qix.c	8.1 AVS 92/10/11";
#endif
/*
			Copyright (c) 1989,1990,1992 by
			Advanced Visual Systems Inc.
			All Rights Reserved
	
	This software comprises unpublished confidential information of
	Advanced Visual Systems Inc. and may not be used, copied or made
	available to anyone, except in accordance with the license
	under which it is furnished.
	
	This file is under sccs control at AVS in:
	/sccs1.p/avs/examples/s.qix.c
	
*/
#include <stdio.h>
#include <avs/avs.h>
#include <avs/field.h>
#include <avs/geom.h>

/******************************************************************************/

/* 
 * This is a C example to create a geometry object.  In this example,
 * the "simulation" program flow of control is used.  Instead of providing
 * a compute module function that is called whenver a parameter or input
 * has changed, this module can determine when it wants to provide new
 * data to the network.  Many existing applications will fit into this 
 * model much more easily than the "compute function" model.
 *   
 * This program is a "string art" program.  It draws disjoint lines in a 
 * random pattern.
 */  

/*  
 * The routine "qix" is the description routine.  It provides
 * AVS some necessary information such as: name, input and output ports, 
 * parameters etc.
 */
qix()
{
	int polygon_compute();	/* declare the compute function (below) */
	int out_port;		/* temporary to hold the port number */
	int parm;		/* temporary to hold the parm number */

	/* Set the module name and type */
	AVSset_module_name("qix", MODULE_DATA);

	/* There are no input ports for this module */

	/* Create an output port for the result */
	out_port = AVScreate_output_port("Output Geometry", "geom");

	/* Add parameter: an enable/disable toggle for the scope */
	parm = AVSadd_parameter("sleep", "boolean", 1, 0, 1);
	/* Add parameter: number of cycles before putting self to sleep */
	parm = AVSadd_parameter("cycles", "integer", 0, 0, 10000);

	/* There is no compute function for this module */
}

#define MAXV 200
#define PERFRAME 6

typedef float FLOAT3[3];

main(argc,argv)
int	argc;
char	*argv[];
{
   int qix();
   int count = MAXV;
   FLOAT3 verts[2], move0, move1, colors[2], movec0, movec1;
   int sleep = 1;
   int cycles = 0,cycle_count=0;
   GEOMobj *obj = NULL;
   GEOMedit_list output = NULL;
   float mat[4][4];
   int i;

/*
 * We initialize this module by passing a pointer to our description
 * function.  The description function is defined below.  It defines our
 * inputs, outputs and parameters.
 * When AVS loads this module, this program is invoked, the description
 * function is called to identify the module (and get the colors on the
 * palette), then the program exits.  When the module is dropped onto
 * the palette, the program is invoked, the description function
 * is called and this function will return.
 */
   AVScorout_init(argc,argv,qix);

   AVScorout_set_sync(1);

   mat_y_rotate(5.0,mat);

   while(1) {
/*
 * If our "sleep" parameter is false, we mark ourselves as having changed.
 * This will cause the subsequent call to AVScorout_wait() to return 
 * right away.  Otherwise, AVScorout_wait will pause until inputs or parameters
 * are changed.
 */
      if (!sleep) AVScorout_mark_changed();

      AVScorout_wait();


/*
 * We get the value of the "sleep" parameter here.  This might cause us to
 * do one extra iteration of the loop but since this is a string art demo
 * it won't matter.  Note that the parameters to AVScorout_input vary
 * based on the number of inputs and parameters that you have.
 */
      AVScorout_input(&sleep,&cycles);

/*
 * For each frame, we compute "PERFRAME" number of lines to add to our
 * object.  We re-use the same object adding lines until we get to the
 * threshold, then destroy the object and start over again.
 */
      for (i = 0; i < PERFRAME; i++) {
	 if (count >= MAXV) {
	    start(verts,colors,move0,move1,movec0,movec1);
	    count = 0;
	    if (obj) GEOMdestroy_obj(obj);
	    obj = GEOMcreate_obj(GEOM_POLYTRI,NULL);
	 }
	 else next(verts,colors,move0,move1,movec0,movec1);
	 GEOMadd_disjoint_line(obj,verts,colors,2,GEOM_COPY_DATA);
	 count++;
      }

/*
 * Once we have an object that we want to display, we create a new
 * edit list.  An edit list contains a list of the changes that we want to
 * make to the scene on any given frame.  We clear out the list (freeing
 * the one from the previous frame), then tell AVS to replace the geometry
 * of an object named 'qix' with the new object.  Then we take the 
 * resulting edit list and output it to AVS.  Note that the parameters
 * to AVScorout_output vary based on the number of outputs that you have.
 */
      output = GEOMinit_edit_list(output);
      GEOMedit_geometry(output,"qix",obj);
      GEOMedit_concat_matrix(output,"qix",mat);
      GEOMedit_camera_params(output,"camera1",GEOM_CAMERA_DEPTH_CUE,1);
      AVScorout_output(output);
      if (cycles && (cycle_count++ > cycles))
          AVSmodify_parameter("sleep",AVS_VALUE,1,0,0);
   }
}

#define RA 5.0
#define DD 0.2
#define DC 0.01

start(verts,colors,move0,move1,movec0,movec1)
FLOAT3 *verts;
FLOAT3 *colors;
FLOAT3 move0, move1;
FLOAT3 movec0, movec1;
{
   float ran();

   verts[0][0] = ran(RA); verts[0][1] = ran(RA); verts[0][2] = ran(RA);
   verts[1][0] = ran(RA); verts[1][1] = ran(RA); verts[1][2] = ran(RA);

   move0[0] = ran(DD); move0[1] = ran(DD); move0[2] = ran(DD);
   move1[0] = ran(DD); move1[1] = ran(DD); move1[2] = ran(DD);

   colors[0][0] = ran(1.0); colors[0][1] = ran(1.0); colors[0][2] = ran(1.0);
   colors[1][0] = ran(1.0); colors[1][1] = ran(1.0); colors[1][2] = ran(1.0);

   movec0[0] = ran(DC); movec0[1] = ran(DC); movec0[2] = ran(DC);
   movec1[0] = ran(DC); movec1[1] = ran(DC); movec1[2] = ran(DC);
}

next(verts,colors,move0,move1,movec0,movec1)
FLOAT3 *verts;
FLOAT3 *colors;
FLOAT3 move0, move1;
FLOAT3 movec0, movec1;
{
   int i;
   for (i = 0; i < 3; i++) {
      verts[0][i] = verts[0][i] + move0[i];
      verts[1][i] = verts[1][i] + move1[i];
      colors[0][i] = colors[0][i] + movec0[i];
      colors[1][i] = colors[1][i] + movec1[i];
      if (verts[0][i] > RA && move0[i] > 0.0) { 
	 verts[0][i] = RA; move0[i] = -move0[i]; 
      }
      if (verts[0][i] < -RA && move0[i] < 0.0) { 
	 verts[0][i] = -RA; move0[i] = -move0[i]; 
      }
      if (verts[1][i] > RA && move1[i] > 0.0) { 
	 verts[1][i] = RA; move1[i] = -move1[i]; 
      }
      if (verts[1][i] < -RA && move1[i] < 0.0) { 
	 verts[1][i] = -RA; move1[i] = -move1[i]; 
      }

      if (colors[0][i] < 0.0) {
	 if (movec0[0] < 0.0) movec0[i] = -movec0[i];
	 colors[0][i] = 0.0;
      }
      if (colors[0][i] > 1.0) {
         if (movec0[i] > 0.0) movec0[i] = -movec0[i]; 
	 colors[0][i] = 1.0; 
      }
      if (colors[1][i] < 0.0) { 
	 if (movec1[1] < 0.0) movec1[i] = -movec1[i]; 
	 colors[1][i] = 0.0; 
      }
      if (colors[1][i] > 1.0) {
	 if (movec1[0] > 0.0) movec1[i] = -movec1[i];
	 colors[1][i] = 1.0; 
      }
   }
}

float
ran(n)
float n;
{
   double drand48();
   return(n * drand48());
}
